; ======================================================================
; $Id: av2_spline.pro,v 1.1 2001/10/22 18:48:46 jlin Exp $
;
; File Description:  This file contains only the procedure av2_spline.
; ======================================================================


;------------------------------------------------------------------
;                        procedure av2_spline
;
; PURPOSE:
;   Takes an array of periodic means, applies a cubic spline in-
;   terpolation to the values, and returns an array of the spline
;   matching the mean, with the spline also being periodic.  In
;   particular, this procedure is used to take a monthly clima-
;   tology and convert it into a smooth, periodic in time daily
;   spline fit of the climatology.
;
; RESTRICTIONS:
;   Currently works only on interpolating a year of monthly means
;   into daily data, with the assumption that the monthly mean
;   represents a point in the middle of the month.  Each month
;   is assumed 30 days long.  Thus, arrayavg gives the monthly
;   mean, assumed to be at the 15th of the month, while
;   arrayinterp gives daily values from day 1 to 360.
;
;   Input and output array variables are either uniformally
;   floating point or double-precision arrays.
;
;   CAUTION:  The more linear the points in arrayavg, the more
;   inadequately the cubic spline interpretation performs.
;
; ALGORITHM:
;   - Find global maximum.
;   - Shift curve so global max. is first point.
;   - Duplicate first point so it's added onto series as the
;     periodic last point.
;   - Spline with first/last point with a slope of 0.
;   - Reshift so a regular single year is returned.
;
; INPUTS AND OUTPUTS:
;   In:   delt               timestep, FLOAT [days]
;         arrayavg(NAVG)     monthly climatology
;   Out:  arrayinterp(NINT)  spline fitted time series
;
;   File I/O:  None
;
; MISCELLANEOUS INFORMATION:
; - For use with output from QTCM1 v2.1.
; - No user-written procedures called within this procedure.
; - No common blocks used in this procedure.
;
; VERSION HISTORY:
; - 10 Jan 2000:  Johnny Lin, UCLA Dept. of Atmospheric Sciences.
;   Based on QTCM1 v2.0 tool av2_spline.pro, v1.1.  Unchanged
;   from that version, which passed reasonable tests 27 Aug 1997.
;------------------------------------------------------------------
PRO AV2_SPLINE, delt,arrayavg,arrayinterp


; <Error checks, size declarations, array initialization>
;
ON_ERROR, 0
SizeAvg=SIZE(arrayavg)  & SizeInt=SIZE(arrayinterp)
DimAvg=SizeAvg(0)       & DimInt=SizeInt(0)
NAVG=SizeAvg(1)         & NINT=SizeInt(1)
TypeAvg=SizeAvg(2)      & TypeInt=SizeInt(2)

if (delt ne 1.0) then MESSAGE, "error-not daily data"
if (DimAvg ne 1) or (DimInt ne 1) then MESSAGE, "error-dimensions wrong"
if (NAVG ne 12) or (NINT ne 360) then MESSAGE, "error-size wrong"
if (TypeAvg ne TypeInt) then MESSAGE, "error-different array types"

TypeArr    = TypeAvg            ;local variable for array type
tmpavg     = dblarr(NAVG)       ;working avg. array a double array
tmpavg1    = dblarr(NAVG+1)     ;working avg. array plus wraparound
tmpinterp  = dblarr(NINT)       ;working interp. array a double array
if (delt eq 1.0) then begin
  tmpinterp1 = dblarr(NINT+30)  ;working interp. array plus wraparound month
endif

CASE TypeArr OF                 ;make working arrays all double-precision
  4: tmpavg = DOUBLE(arrayavg)        ;single-precision case
  5: tmpavg = arrayavg                ;double-precision case
  else: MESSAGE, "error-array types not single or double floating"
ENDCASE


; <Calculate spline>
;
maxavg = MAX(tmpavg,maxloc)            ;find global maximum
tmpavg = SHIFT(tmpavg,-maxloc)         ;shift curve so global max. is 1st pt.
tmpavg1(0:NAVG-1) = tmpavg(0:NAVG-1)   ;duplicate 1st pt. as periodic endpt.
tmpavg1(NAVG)     = tmpavg(0)

if (delt eq 1.0) then begin
  xint1 = DOUBLE(FINDGEN(NINT+30)+1.0)               ;day no. tmpinterp1 elem.
  xavg1 = DOUBLE(((FINDGEN(NAVG+1)+1.0)*30.0)-15.0)  ;day no. tmpavg1 elem.
endif

Result = SPL_INIT(xavg1,tmpavg1,yp0=0,ypn_1=0,/double)      ;spline w/ 1st &
tmpinterp1 = SPL_INTERP(xavg1,tmpavg1,Result,xint1,/double) ;last pt. slope=0

if (delt eq 1.0) then begin
  tmpinterp(0:NINT-1) = tmpinterp1(14:14+NINT-1)   ;select one year
  tmpinterp = SHIFT(tmpinterp,(maxloc*30)+14)      ;reshift for regular year
endif


; <Explicitly make output correct for given precision>
;
CASE TypeArr OF
  4: arrayinterp = FLOAT(tmpinterp)      ;single-precision case
  5: arrayinterp = tmpinterp             ;double-precision case
ENDCASE

RETURN
END     ;=== end of procedure av2_spline ===


; ========== end av2_spline.pro file ==========
