;------------------------------------------------------------------
; $Id: calcanomL_dly.pro,v 1.1 2002/06/23 03:49:03 jlin Exp jlin $
;+
;                        procedure calcanom
;
; PURPOSE:
;   Calculate dataset anomalies assuming mean of each day of the
;   year (i.e. all Jan 1's, all Jan 2's) over the entire timeseries
;   is the climatology.  All dimensions are LONG integer.
;
; RESTRICTIONS:
;   - This program assumes each month has 30 days in it.
;   - Data must be daily.
;   - Data must start at the beginning of the calendar year.
;
; INPUTS AND OUTPUTS:
;   In:   delt, data(NX,NY,NT) (full data on entry)
;   Out:  data (altered to contain anomalies on exit)
;
;   File I/O:  None
;
; MISCELLANEOUS INFORMATION:
; - For use with QTCM1 v2.1 output
; - All arrays are defined as double-precision
; - No user-written procedures required.
; - No common blocks used in this procedure
;
; CODE HISTORY:
; - 22 Jun 2002:  By Johnny Lin, CIRES/University of Colorado.
;   Email:  air_jlin@yahoo.com.  Passed moderately reasonable 
;   tests.
;-
; Copyright (c) 2002 Johnny Lin.  For licensing, distribution 
; conditions, and contact information, see the following page:
; http://www.johnny-lin.com/lib.html.
;------------------------------------------------------------------

PRO CALCANOM, delt, data



; ---------------------- global error check -----------------------
;
if (delt ne 1.0) then MESSAGE, 'error--delt bad value'
sizeinfo = SIZE(data)
NX = sizeinfo(1) & NY = sizeinfo(2) & NT = sizeinfo(3)

if (delt eq 1.0) and ((NT mod 360) ne 0) then       $
   MESSAGE, 'error--not integral multiple of yrs.'
if (delt eq 30.0) and ((NT mod 12) ne 0) then       $
   MESSAGE, 'error--not integral multiple of yrs.'
if ((delt eq 1.0) and (NT lt 360)) or               $
   ((delt eq 30.0) and (NT lt 12)) then             $
   MESSAGE, 'error--too few time pts.'

if (sizeinfo(4) ne 5) then MESSAGE, 'error--data not double floating'



; ------------- begin loop through each spatial point -------------

PRINT, '  -> Calculate spline-fit climatology and anomalies'

for iy=0L,NY-1 do begin
    for ix=0L,NX-1 do begin
        data_xy = REFORM(data[ix,iy,*])   ;- timeseries at this x-y pt



; --------------------- calculate climatology ---------------------
;
; Loop through each day of a year to calculate the climatology at
; this point for one year.  Then concatenate to expand this climatol.
; until you reach the full length of the timeseries.  Output of this
; section is clim_xy.

max_nyrs = CEIL(FLOAT(NT)/360.0)       ;- # of yrs in timeseries, rounded up
base_1yr_pts = LINDGEN(max_nyrs)*360   ;- loc of all day 0's in timeseries

clim_1yr = DBLARR(360)                 ;- init. climatology for 1 yr array

for idy=0,360-1 do begin
    all_idy_pts = base_1yr_pts + idy               ;+ loc of all idy points

    good_pts = WHERE(all_idy_pts le NT-1, count)   ;+ choose only pts that
    if (count gt 0) then  $                        ;  aren't longer than
       all_idy_pts = all_idy_pts[good_pts]         ;  the timeseries length

    clim_1yr[idy] = MEAN(data_xy[all_idy_pts])     ;+ climatol. at idy
endfor

clim_xy = clim_1yr                  ;- concatenate over all years in
for iyrs=2,max_nyrs do begin        ;  timeseries and truncate to fit
    clim_xy = [clim_xy, clim_1yr]   ;  timeseries length NT
endfor
clim_xy = clim_xy[0:NT-1]



; ---------------------- calculate anomalies ----------------------

anom = TEMPORARY(data_xy) - TEMPORARY(clim_xy)
data[ix,iy,*] = TEMPORARY(anom)


 
; -------------- end loop through each spatial point --------------

    endfor
endfor



RETURN
END     ;=== end of procedure calcanom ===


; ========== end calcanomL_dly.pro file ==========
