; ======================================================================
; $Id: calcanomL_spl.pro,v 1.1 2002/06/23 03:49:03 jlin Exp jlin $
;
; File Description:  This file contains only the procedure calcanom.
; ======================================================================
 
 
;------------------------------------------------------------------
;                        procedure calcanom
;
; PURPOSE:
;   Calculate dataset anomalies from monthly climatology.  Spline
;   smoothing applied to climatology for the case of daily data.
;   All dimensions are LONG integer.
;
; RESTRICTIONS:
;   - This program assumes each month has 30 days in it.
;   - Data must be either daily or monthly.
;   - Data must start at the beginning of the calendar year, and
;     last through integral multiple of years.
;
; INPUTS AND OUTPUTS:
;   In:   delt, data(NX,NY,NT) (full data on entry)
;   Out:  data (altered to contain anomalies on exit)
;
;   File I/O:  None
;
; MISCELLANEOUS INFORMATION:
; - For use with QTCM1 v2.1 output
; - All arrays are defined as double-precision
; - User-written procedures required:  av2_spline
; - No common blocks used in this procedure
;
; CODE HISTORY:
; - 10 Jan 2000:  Johnny Lin, UCLA Dept. of Atmospheric Sciences.
;   Based on QTCM1 v2.0 tool calcanomL.pro, v1.3.
;------------------------------------------------------------------

PRO CALCANOM, delt, data


; <global error check>
;
if (delt ne 1.0) and (delt ne 30.0) then MESSAGE, 'error--delt bad value'
sizeinfo = SIZE(data)
NX = sizeinfo(1) & NY = sizeinfo(2) & NT = sizeinfo(3)

if (delt eq 1.0) and ((NT mod 360) ne 0) then       $
   MESSAGE, 'error--not integral multiple of yrs.'
if (delt eq 30.0) and ((NT mod 12) ne 0) then       $
   MESSAGE, 'error--not integral multiple of yrs.'
if ((delt eq 1.0) and (NT lt 360)) or               $
   ((delt eq 30.0) and (NT lt 12)) then             $
   MESSAGE, 'error--too few time pts.'

if (sizeinfo(4) ne 5) then MESSAGE, 'error--data not double floating'



; <calculate climatology and anomalies>
;
PRINT, '  -> Calculate spline-fit climatology and anomalies'

nclimpts = dblarr(NX,NY,12)    ;local: # pts. used to calc. each mo. climatol.
clim = dblarr(NX,NY,12)        ;local: climatology array of monthly means
nclimpts(*,*,*) = 0.d0         ;zero global local arrays
clim(*,*,*) = 0.d0


;     For daily data (begin)
;^^^^^
if (delt eq 1.0) then begin    ;* for daily data

  idy=0                        ;climatology
  imo=1
  nclimpts(*,*,*) = 0.d0
  for it=0L,NT-1 do begin
    idy=idy+1
    clim(*,*,imo-1)=clim(*,*,imo-1)+data(*,*,it)
    nclimpts(*,*,imo-1)=nclimpts(*,*,imo-1)+1.d0
    if (idy eq 30) then begin
       imo=imo+1
       idy=0
       if (imo eq 13) then imo=1
    endif
  endfor

  for imo=1,12 do begin            ;calc. monthly means
    clim(*,*,imo-1) = clim(*,*,imo-1) / nclimpts(*,*,imo-1)
  endfor
  climdly1 = dblarr(NX,NY,360)     ;convert moly means to daily data
  for iy=0L,NY-1 do begin
    for ix=0L,NX-1 do begin
      tmpclim=dblarr(12) & tmpclimdly1=dblarr(360)
      tmpclim(*) = clim(ix,iy,*)
      AV2_SPLINE, delt,tmpclim,tmpclimdly1
      climdly1(ix,iy,*) = tmpclimdly1(*)
    endfor
  endfor

  idy=0                        ;anomalies
  for it=0L,NT-1 do begin
    idy=idy+1
    data(*,*,it) = data(*,*,it) - climdly1(*,*,idy-1)
    if (idy eq 360) then idy=0
  endfor

endif                          ;* for daily data
;^^^^^For daily data (end)


;     For monthly data (begin)
;^^^^^
if (delt eq 30.0) then begin   ;* for monthly data

  imo=1                        ;climatology
  nclimpts(*,*,*)=0.d0
  for it=0L,NT-1 do begin
    clim(*,*,imo-1)=clim(*,*,imo-1)+data(*,*,it)
    nclimpts(*,*,imo-1)=nclimpts(*,*,imo-1)+1.d0
    imo=imo+1
    if (imo eq 13) then imo=1
  endfor

  for imo=1,12 do begin
    clim(*,*,imo-1) = clim(*,*,imo-1) / nclimpts(*,*,imo-1)
  endfor

  imo=1                        ;anomalies
  for it=0L,NT-1 do begin
    data(*,*,it) = data(*,*,it) - clim(*,*,imo-1)
    imo=imo+1
    if (imo eq 13) then imo=1
  endfor

endif                          ;* for monthly data
;^^^^^For monthly data (end)

RETURN
END     ;=== end of procedure calcanom ===


; ========== end calcanomL.pro file ==========
