; ==========================================================================
; $Id: grid_interp.pro,v 1.1 2001/10/22 18:48:46 jlin Exp $
;
; File Description:  This file contains only the function grid_interp.
; ==========================================================================


;-------------------------------------------------------------------------
;                            function grid_interp
;
; Purpose:
;   Interpolates half-indice grid point to non-half indice grid points.
;   Primarily this function is used to either interpolate QTCM1 u and
;   v points to the same points as T.  See figure 2.1 in the QTCM v2.1
;   manual (an Arakawa staggered C-grid).
;
;   Note:  IDL grid indexes [0:NX-1, 0:NY-1]
;          QTCM numerical output grid indexes [1:NX, 1:NY]
;          C-grid can index by half-indice grid points (or not)
;
;   Interpolation is not done on the basis of statute distance, but is
;   just a regular two-point interpolation.
;
;   NB:  Currently, function doesn't accept vorticity and streamfctn.
;   locations for interpolation.
;
; Parameters In:
;   typeflag         Flag describing location of the grid point in 
;                    gridin (string):
;                      'u' = gridin is for points at u locations,
;                            gridin[0,j] is the same as [1,j] on the
;                            QTCM1 numerical output grid, which is
;                            the same as [1.5,j] on the C-grid.
;                      'v' = gridin is for points at v locations,
;                            gridin[i,0] is the same as [i,1] on the
;                            QTCM1 numerical output grid, which is
;                            the same as [i,1.5] on the C-grid.
;                      'T' = gridin is at temp. pts.; no interpol.
;                            is needed.
;   gridin[NX,NY]    Input grid (any numerical type; converted to
;                    DOUBLE in the program for calculations).
;
; Parameters Out:
;   gridout[NX,NY]   Input grid (double float).  All array dimensions
;                    are type long.  Values are at the locations of
;                    the T points.  Created.
;
; File I/O:  None
;
; Notes:
; - NB:  *** NX and NY must span the full QTCM1 grid (esp. NX, which
;   must go all the way around the globe in longitude).  To help make
;   sure this occurs, currently this function checks that NX=64.
; - The del x and del y of the grid are assumed to be regular.
; - All calculations done in double precision.
; - No user-written procedures used.
; - No common blocks used in this procedure.
; - Written in IDL 5.0.
;
; Revision History:
; - 4  Jan 2000:  Orig. by Johnny Lin, UCLA Dept. of Atmos. Sci.  Passed
;                 resonable tests.
; - 18 Jan 2000:  Add case of 'Not Given' typeflag.
; - 20 Jan 2000:  Small bug fixed for T-grid and Not Given case.
;-------------------------------------------------------------------------

FUNCTION GRID_INTERP, typeflag, gridin


;             --- parameter initializing and error checking ---

On_Error, 0
if (N_PARAMS() ne 2)          then Message, 'error--missing arg'
if (ARG_PRESENT(gridin) eq 0) then Message, 'error--missing arg'

sizeinfo = SIZE(gridin)
if (sizeinfo[0] ne 2) then Message, 'error--data dims bad'
NX = sizeinfo[1]    ;SIZE returns LONG dimensions
NY = sizeinfo[2]    ;SIZE returns LONG dimensions

if (NX lt 2) or (NY lt 2) then Message, 'error--NX,NY too small'
if (NX ne 64) then Message, 'error--possible NX problem grid_interp'



;                             --- calculate ---

Print, '  -> Grid interpolation from points at '  $
       + STRTRIM(typeflag,2) + '-variable location'

tmpgrid_phalf      = DOUBLE(gridin)    ;phalf = plus half-index
tmpgrid_mhalf      = DBLARR(NX,NY)     ;mhalf = minus half-index
tmpgrid_mhalf[*,*] = !VALUES.F_NAN     ;set to NaN as a checking mechanism


case typeflag of      ; -> begin typeflag case

  'u': begin                ; -> u-grid case
      Print, '     - x-boundary assumed periodic'
      tmpgrid_mhalf[1:NX-1,*] = tmpgrid_phalf[0:NX-2,*]
      tmpgrid_mhalf[0,*]      = tmpgrid_phalf[NX-1,*]
      gridout = (tmpgrid_mhalf + tmpgrid_phalf) / 2.d0
    end


  'v': begin                ; -> v-grid case
      Print, '     - interpol. y-bott. bound. set equal to 1st y-element'
      tmpgrid_mhalf[*,1:NY-1] = tmpgrid_phalf[*,0:NY-2]
      tmpgrid_mhalf[*,0]      = tmpgrid_phalf[*,0]
      gridout = (tmpgrid_mhalf + tmpgrid_phalf) / 2.d0
    end


  'T': begin                ; -> T-grid case
      Print, '     - no interpol. needed'
      gridout = gridin
    end

  'Not Given': begin        ; -> Not Given case
      Print, '     - Not Given typeflag:  Default no interpol.'
      gridout = gridin
    end

  else: Message, 'error--bad typeflag'

endcase               ; -> end typeflag case



return, gridout
end     ;=== end function ===


; ========== end file ==========
