;+
; $Id: main.pro,v 1.1 2001/10/22 18:48:46 jlin Exp $
;
; PURPOSE:
;     Main program to calculate zonal wavenumber (k) vs. frequency (w)
;       spectral power on anomalies.  Operates on a variable field from
;       a QTCM output file.  There is no GUI for this program.
;
;     This program is suitable for QTCM version 2.1, and higher.  This
;       program reads in raw, full QTCM1 output data, and then computes
;       the anomalies, then computes PSD.
;
;     Lines defined as USER ADJUSTABLE need to be correct for the program
;       to work.
;
;     Based on rev. 1.5 of main_qfft.kfn_w.pro.
;
; IN:
;     fname            QTCM output file
;
; OUT:
;     spectra.out      Output file of spectral density array
;     freq.out         Output file of frequency array
;     k.out            Output file of wavenumber array
;
; NOTES AND RESTRICTIONS:
; - NB:  data and spectra are defined DBLARR.  All other global
;   arrays are FLTARR.
; - All required user-defined procedures are located in this file.
; - First record of dataset must be either Jan 1st for daily data,
;   or Jan for monthly data.
; - Requires QTCM output to have periodic but non-ghostpointed boundary
;   conditions in the zonal direction.
; - Plotting logic assumes default device is initially set to the
;   screen when the program starts running.
; - For information on calculating power spectrums from FFTs, see Press
;   et al., Numerical Recipies in Pascal, 1989, p. 465.  Briefly, FFT
;   is first performed spatially to obtain FFT coefficients for zonal
;   wavenumber.  Then FFT is performed on the wavenumber coefficients
;   with the appropriate windowing to obtain a periodogram estimator
;   for the frequency domain.  PSD standard deviations are reduced
;   using the method of Press, et al., described at the bottom of
;   page 465, to preserve the normalization property.  Note my power
;   estimator differs from Press et al. in that power is defined for
;   negative frequencies.
; - Common blocks:
;     main_com   Key main panel user adjustable parameters and parameters
;                related to FFT computation
; - User-written procedures required:  See batch file qfft.kfn.pro for
;   list.
;
; CODE HISTORY:
; - 10 Jan 2000:  Orig. by Johnny Lin, UCLA Department of Atmospheric
;   Sciences.  Based on QTCM1 v2.0 tool main_qfft.kfn.pro, v1.3.  Main
;   program checked for compatibility with QTCM1 v2.1 on 11 Jan 1999.
;   Passed minimally reasonable tests of consistency with QTCM1 v2.0
;   tool.
; - 18 Feb 2000:  Changes to allow automated running.
; - 5 Jun 2000:  Added code to detrend using least squares fitting, where
;   you minimize the chi-square error and remove the mean at each longit.
;   point in CALCFVSK before calculating the spectra.  Main code insert
;   passed reasonable tests.
; - 22 Oct 2001:  "Public release" version.
;-----------------------------------------------------------------------------



; +==========================================================================+
; |                       GENERAL AND TASK PROCEDURES                        |
; +==========================================================================+


;------------------------------------------------------------------
;                      procedure errorcheck
;
; Purpose:  Error check to do before any calculation or plotting.
;
; In:   NT, delt
; Out:  None
;
; Johnny Lin / Department of Atmospheric Sciences / April 1997
; Checked for compatibility with QTCM1 v2.1 on 10 Jan 1999
;
; No procedures called by this procedure
; No common blocks in this procedure
;------------------------------------------------------------------
PRO ERRORCHECK, NT, delt

ON_ERROR, 0
NF = LONG(FLOAT(NT)/2.0)           ;number of pos. (non-zero) freq. pts.
if (LONG(NF*2) ne NT) then MESSAGE, 'error--NT,NF mismatch means odd NT'
if (delt ne 1.0) and (delt ne 30.0) then MESSAGE, 'error--delt uncovered value'

RETURN
END     ;=== end of procedure errorcheck ===



;------------------------------------------------------------------
;                         procedure binkinfo
;
; Purpose:  For given bink, NF, and NT, tells how many freq-
;           ency points there will be once all the freqencies
;           have been binned (ival), and which frequencies will
;           remain after the binning (=1 in array freqinfo means
;           that that frequency remains).
;
; In:   bink,NF,NT
; Out:  freqinfo,ival
;
; Note that both freq and freqinfo are "pre-shifted," i.e. that
;   the array runs from [0,+Nyquist], then [+Nyquist-1/Ndelt,
;   1/Ndelt].
;
; Johnny Lin / Department of Atmospheric Sciences / June 1997
; Procedure passed reasonably adequate tests 9 Jun 1997.
; Checked for compatibility with QTCM1 v2.1 on 10 Jan 1999
;
; No procedures or functions called by this function
; No common blocks in this function
;------------------------------------------------------------------
PRO BINKINFO, bink,NT,NF, freq_remain,ival

; <error checks>
;
ON_ERROR, 0
sizeinfo = SIZE(freq_remain) & if (sizeinfo(1) ne NT) then MESSAGE,'err-fr NT'
if (NF ne LONG(FLOAT(NT)/2.0)) then MESSAGE, 'err--NT,NF mismatch'
if (LONG(NF*2) ne NT) then MESSAGE, 'err--NT,NF mismatch means odd NT'
if (FLOAT(LONG(FLOAT(bink)/2.0)) eq FLOAT(bink)/2.0) or (bink lt 1) then $
  MESSAGE, 'err--bink even or too small'


; <figure out which frequencies will remain after binning>
;
tmpfreq_remain = lonarr(NT)    ;array showing which freq. remain after binning
tmpfreq_remain(*) = 0          ;I.C. tmpfreq_remain all don't remain (=0)
binkd2 = LONG(FLOAT(bink)/2.0)  ;"half" a bink

i=0 & tmpfreq_remain(i)=1 & dist_since_last_remain=0    ;zero/pos. freq.
for i=1L,NF do begin
  dist_since_last_remain = dist_since_last_remain + 1
  if (dist_since_last_remain eq bink) then begin
     if ((i+binkd2) le NF) then tmpfreq_remain(i)=1
     dist_since_last_remain = 0
  endif 
endfor

dist_since_last_remain = 0                              ;neg. freq.
for i=NT-1,NF+1,-1 do begin
  dist_since_last_remain = dist_since_last_remain + 1
  if (dist_since_last_remain eq bink) then begin
     if ((i-binkd2) ge (NF+1)) then tmpfreq_remain(i)=1
     dist_since_last_remain = 0
  endif 
endfor

; <count occurences of remaining frequencies>
;
tmpival = 0
for i=0L,NT-1 do if (tmpfreq_remain(i) eq 1) then tmpival=tmpival+1

freq_remain(*) = tmpfreq_remain(*)     ;output values
ival = tmpival

RETURN
END     ;=== end of procedure binkinfo ===



;------------------------------------------------------------------
;                         procedure calcfvsk
;
; Purpose:  Calculates the power spectra with respect to freqency
;           for various wavenumbers (k).
;
; File I/O:  None
;
; In:   data(NX,NY,NT), lat, freq_remain(NT)
; Out:  binspectra(NK,NTB), xvector(NK), yvector(NTB)
;
; When this procedure exits, binspectra and yvector will be
;   arranged so that frequency runs monotonically from +Nyquist
;   to -Nyquist.  Note also that the programs returns a binned
;   number of frequency points (NTB).
;
; NB:  data & binspectra is defined DBLARR.  Other input arrays
;   are FLTARR.
;
; Keywords:
;   MEAN_AND_DETREND   If true, then remove mean and detrend
;     latavg at each longit. point.
;
; 11 Jan 2000:  Changes added to enable some dimensions be LONG.
; Checked for compatibility with QTCM1 v2.1 on 11 Jan 1999
;
; Procedures called:  FFT, HANNING, etc.
; Common block:  main_com
;------------------------------------------------------------------
PRO CALCFVSK, data,lat,freq_remain, binspectra, xvector, yvector  $
            , MEAN_AND_DETREND = mean_and_detrend

COMMON main_com, NX,NY,NT,delt,nlat,slat,kfix,alpha,bink

; <error checking>
;
ON_ERROR, 0
ERRORCHECK, NT, delt
sizeinfo = SIZE(binspectra) & NK = sizeinfo(1) & NTB = sizeinfo(2)
sizeinfo = SIZE(yvector) & if (sizeinfo(1) ne NTB) then MESSAGE,'err-NTB y'
sizeinfo = SIZE(freq_remain) & if (sizeinfo(1) ne NT) then MESSAGE,'err-NT fr'
sizeinfo = SIZE(data) & if (sizeinfo(3) ne NT) then MESSAGE,'err-NT d'
if (NTB gt NT) then MESSAGE,'err-NTB too big'

if (nlat lt slat) then MESSAGE, 'error--nlat lt slat'
if (NY gt 9990) then MESSAGE, 'error--NY too big'
ntest = LONG(FLOAT(NX)/2.0)      ;approx. half zonal wavenumber elements
if (NK ge ntest) then MESSAGE, 'error--too many zonal wavenumbers'
if (FLOAT(LONG(FLOAT(bink)/2.0)) eq FLOAT(bink)/2.0) or (bink lt 1) then $
  MESSAGE, 'error-bink even or too small'

tmpspectra = dcomplexarr(NK,NT)            ;unbinned spectra array


; <find latitudes for which to average>
;
latused=lonarr(NY)
latused(*)=-9999
for ilat=0L,NY-1 do $
  if (lat(ilat) le nlat) and (lat(ilat) ge slat) then latused(ilat)=ilat
icount=0
for ilat=0L,NY-1 do if (latused(ilat) eq -9999) then icount=icount+1
if (icount eq NY) then MESSAGE, 'error--latitude range too small'

inlat = MAX(latused)       ;ilat value of N boundary of range
for ilat=0L,NY-1 do if (latused(ilat) eq -9999) then latused(ilat)=9999
islat = MIN(latused)       ;ilat value of S boundary of range


; <average latitude>
;
latavg = dblarr(NT,NX)
for ilon=0L,NX-1 do begin
  for it=0L,NT-1 do begin
    icount=0
    avg=0.d0
    for ilat=islat,inlat do begin
      avg = avg + data(ilon,ilat,it)
      icount = icount + 1
    endfor
    latavg(it,ilon) = avg / DOUBLE(icount)
  endfor
endfor


; <remove mean and detrend at each longit. point>
;
if (KEYWORD_SET(mean_and_detrend) ne 0) then begin
   for ilon=0L,NX-1 do begin
     tmpnt  = REFORM(latavg[*,ilon])
     tmpnt  = tmpnt - MEAN(tmpnt)             ;remove mean

     tmpidx = DINDGEN(NT)
     coeff  = LINFIT(tmpidx, tmpnt, /DOUBLE)
     tmpfit = coeff[0] + (coeff[1] * tmpidx)
     tmpnt  = tmpnt - tmpfit                  ;detrend

     latavg[*,ilon] = tmpnt[*]
   endfor
endif


; <compute zonal wavenumber array>
;
kspectra = dcomplexarr(NT,NK)
tmplatavg = dcomplexarr(NT,NX)
tmplatavg = DCOMPLEX(latavg)
tmpk = dcomplexarr(NX)
tmpwink = dblarr(NX)
tmpwink = HANNING(NX, ALPHA=alpha)
for it=0L,NT-1 do begin
  tmpk(*) = DCOMPLEX(tmpwink(*)) * tmplatavg(it,*)
  tmpk = FFT(tmpk, 1, /OVERWRITE, DOUBLE=1)      ;set to inverse transform
  kspectra(it,0:NK-1) = tmpk(0:NK-1)
endfor


; <compute frequency spectra for wavenumbers zero to NK-1>
;
tmpf = dcomplexarr(NT)
tmpwinf = dblarr(NT)
tmpwinf = HANNING(NT, ALPHA=alpha)

for ik=0L,NK-1 do begin
  tmpf(*) = DCOMPLEX(tmpwinf(*)) * kspectra(*,ik)
  tmpf = FFT(tmpf, 1, /OVERWRITE, DOUBLE=1)      ;set to inverse transform
  tmpspectra(ik,*) = tmpf(*)
endfor

tmpwinf(*) = tmpwinf(*)^2.0
Wss = DOUBLE(NT) * TOTAL(tmpwinf, /DOUBLE)
if (Wss eq 0.d0) then MESSAGE, 'error--Wss zero fvsk'
tmpspectra(*,*) = (ABS(tmpspectra(*,*))^2.0) / Wss  ;power spectrum algorithm
                                                    ;partly based on Press et
                                                    ;al., 1989, p. 465-7


; <make spectra so freq. goes continuously from -Nyquist to +Nyquist>
;
NF = LONG(FLOAT(NT)/2.0)                ;# of pos. (non-0) full freq. pts.
freq = fltarr(NT)
for i=0L,NF do freq(i)=FLOAT(i)/(FLOAT(NT)*delt)   ;fill full freq. vector
for i=NT-1,NF+1,-1 do freq(i)=FLOAT(i-NT)/(FLOAT(NT)*delt)

tmpspectra = SHIFT(tmpspectra,0,NF-1)    ;shift power so continuous
freq = SHIFT(freq,NF-1)                  ;shift freq. axis so continuous
freq_remain = SHIFT(freq_remain,NF-1)    ;shift freq_remain so continuous


; <bin spectra to improve variance of PSD estimate>
;
binkd2 = LONG(FLOAT(bink)/2.0)        ;"half" a time bink
for ik=0L,NK-1 do begin              ;go through each wavenumber
  j = 0                              ;which freq. remain after binning
  for it=0L,NT-1 do begin             ;go through each f:  grp. sum, not avg.
    if (freq_remain(it) eq 1) then begin
      yvector(j) = freq(it)
      binspectra(ik,j)=0.0 & ilo=it-binkd2 & ihi=it+binkd2
      for k=ilo,ihi do binspectra(ik,j)=binspectra(ik,j)+tmpspectra(ik,k)
      j = j+1
    endif
  endfor
endfor
if (j ne NTB) then MESSAGE, 'error-j NTB mismatch'


; <create k array and make freq. go from +Nyquist to -Nyquist>
;
for i=0L,NK-1 do xvector(i)=FLOAT(i)    ;create wavenumber array
yvector(*) = yvector(*) * (-1.0)       ;apply factor of neg. 1 to freq.
                                       ;  axis to make sure that we'll have
                                       ;  w/t>0 give eastward propagation

RETURN
END     ;=== end of procedure calcfvsk ===




; +==========================================================================+
; |                             EXECUTE MAIN                                 |
; +==========================================================================+

COMMON main_com, NX,NY,NT,delt,nlat,slat,kfix,alpha,bink



; ----------------------- (begin) USER ADJUSTABLE -----------------------

fname = 'qmean.a0s.r13a.prec.T.16200.s1.out'  ;QTCM input data filename
field = 'prec'             ;field to examine
NX = 64L                   ;longitude array length
NY = 4L                    ;latitude array length
NT = 16200L                ;number of time steps
delt = 1.0                 ;QTCM output file time step increment [day]
NFIELD = 1                 ;total number of fields at each timestep in file
NTSKIP = 0                 ;number of first timesteps to skip
baselat = -5.625           ;lat. of S bound. of dataset domain [deg]
dellat = 3.75              ;lat. increment [deg]

nlat = 8.0                 ;N lat. boundary for lat. band to compute PSD
slat = -8.0                ;S lat. boundary for lat. band to compute PSD
alpha = 0.5                ;windowing function alpha (Hanning alpha=0.5)
bink = 101                 ;bin length

NK = 21                    ;max. # of wavenumbers (do not change from 21)
interp_grid = 'n'          ;n = force no grid_interp in Readdata; y = do
                           ;  grid_interp in Readdata if needed

mean_and_detrend_ilon = 'y'  ;y = CALCFVSK will remove the mean and detrend
                             ;  at every longit. pt., after latit. band is
                             ;  averaged; n = mean and trend not removed
; ----------------------- (end) USER ADJUSTABLE -----------------------



; -------- Other Parameters, Array Declarations, Selected Error Check --------

; <key array declaration>
;
data = dblarr(NX,NY,NT)        ;data array (first for full data, then anom.)
lat = fltarr(NY)               ;latitude array
spectra = dblarr(NT,NY)        ;power spectra.
xvector = fltarr(NT)           ;frequency vector in range [+/- Nyquist]
yvector = fltarr(NY)           ;latitude vector


; <selected error check>
;
if (nlat lt slat) then MESSAGE, 'error-nlat lt slat'



; -------------------------- Read QTCM Raw Data --------------------------

ERRORCHECK, NT, delt
PRINT, '*** Read QTCM data and calc. anom.'
if ((delt eq 1.0) and (NT lt 360)) or   $
   ((delt eq 30.0) and (NT lt 12)) then $
   MESSAGE, "error-unable to calculate anomalies"
data = 0.d0                 ;dealloc. memory to conserve
data = dblarr(NX,NY,NT)     ;reinit. data array
lat = fltarr(NY)            ;reinit. lat array

case interp_grid of
  'n':  READDATA, fname,field,NFIELD,delt,NTSKIP, data, /FORCE_NO_GRID_INTERP
  'y':  READDATA, fname,field,NFIELD,delt,NTSKIP, data
  else:  Message, 'error--bad interp_grid'
endcase

CALCANOM, delt, data
for i=0L,NY-1 do lat(i)=baselat+(FLOAT(i)*dellat)  ;fill latit. vector



; ----------------------------- Calculate PSD -----------------------------

; <calculations>
;
PRINT, '*** Start calc. f vs. 21 k for latitude range'
NF = LONG(FLOAT(NT)/2.0)      ;# of pos. (non-zero) freq. pts.
freq_remain = lonarr(NT)     ;which freq. remain after binning
BINKINFO, bink,NT,NF, freq_remain,ival

NTB=ival                     ;number of binned freq. est.
spectra = dblarr(NK,NTB)     ;binned PSD values
xvector = fltarr(NK)         ;wavenumbers solved for
yvector = fltarr(NTB)        ;binned freq. values
case mean_and_detrend_ilon of
  'y':  CALCFVSK, data,lat,freq_remain, spectra, xvector, yvector  $
                , /MEAN_AND_DETREND
  'n':  CALCFVSK, data,lat,freq_remain, spectra, xvector, yvector
  else: MESSAGE, 'error--bad mean and detrend flag'
endcase


; <diagnostics and plot preparation>
;
minspectra=MIN(spectra) & maxspectra=MAX(spectra)
print, "    -> Spectra min. = " + STRING(minspectra) + "."
print, "    -> Spectra max. = " + STRING(maxspectra) + "."


PRINT, '*** End calc. f vs. k'



; -------------------------- Output PSD as Files --------------------------

ERRORCHECK, NT, delt
print, '*** Write spectra, etc. out to files'

openw, nunit, 'spectra.out', /GET_LUN          ;output spectra power
  printf, nunit, spectra, FORMAT='(E13.6)'
FREE_LUN, nunit

openw, nunit, 'k.out', /GET_LUN                ;output wavenumbers
  printf, nunit, xvector, FORMAT='(E13.6)'
FREE_LUN, nunit

openw, nunit, 'freq.out', /GET_LUN             ;output frequencies
  printf, nunit, yvector, FORMAT='(E13.6)'
FREE_LUN, nunit

PRINT, '*** End qfft.kfn program'



END     ;=== end of main program ===


; ========== end file ==========
