; ==========================================================================
; $Id: readdataL.pro,v 1.1 2001/10/22 18:48:46 jlin Exp $
;
; File Description:  This file contains only the procedure readdata.
; ==========================================================================


;-------------------------------------------------------------------------
;                           procedure readdata
;
; Purpose:
;   Read in single-precision floating point QTCM output data.  Able to
;   read in variables output from QTCM, and some secondary variab.  Can
;   read in either a single timeslice or multiple timeslices.
;
;   Secondary fields are of the form:  const1*var1 + const2*var2
;
;   This allows calculation of us, u900, vs, q_T (i.e. q-T), etc.
;   to be done.
;
;   NB1:  A non-secondary field will not execute grid_interp, if a
;   Varlist is used that does not return a gridloctype of 'u', 'v',
;   or 'T' for that field.
;
;   NB2:  Be aware whether or not the dataset you're using this procedure
;   to read has already been interpolated onto the T grid.  If so, make
;   sure you use the FORCE_NO_GRID_INTERP	keyword, otherwise you run the
;   risk of interpolating it again.
;
; Parameters In:
;   fname       Filename (string)
;   field       Variable name (string)
;   NFIELD      No. of variables outputted by model (integer)
;   delt        Time step length, in days (float)
;   NTSKIP      No. timesteps to skip (integer)
;
; Parameters Out:
;   data[NX,NY] or data[NX,NY,NT]:
;               Data array (double float).  All array dimensions are
;               type long.  NT is number time steps, NX and NY are the
;               number elements in x and y-directions, respectively.
;               If necessary, this procedure will interpolate data to
;               the "T" grid, using a linear, non-distance weighted
;               average.
;
; Keywords:
;   IUNIT_OPEN  This procedure can be used to read grids from a unit
;               that is already opened.  If this keyword is defined,
;               the value it is set to is set equal to iunit in the
;               procedure; fname is ignored.
;
;   FORCE_NO_GRID_INTERP	If keyword is defined, then no grid
;                             interpolation will occur.  You must
;                             set this if your datafile contains
;                             values already interpolated to the T
;                             grid, otherwise you run the risk of
;                             interpolating it again.
;
; File I/O:
;   If IUNIT_OPEN is present, there is no file i/o; iunit_open is
;                 assumed to be opened and closed outside this
;                 procedure.  The file format of iunit_open is
;                 assumed to be /F77_UNFORMATTED.
;   If IUNIT_OPEN is not defined, fname opened and closed within
;                 procedure as iunit.
;
; Notes:
; - User-written procedures used:  Varlist, Vartypeit, GRID_INTERP
; - No common blocks used in this procedure.
; - Works with QTCM v2.1 standard output.
; - Written in IDL 5.0.
;
; Revision History:
; - 29 Dec 1999:  Orig. by Johnny Lin, UCLA Dept. of Atmos. Sci.  Based
;                 on QTCM1 v2.0 tools readdataL.pro RCSv1.8.  Passed
;                 moderately reasonable tests.  Haven't explicitly
;                 tested that this procedure uses convmult correctly,
;                 since the test varlist uses convmult=1.
; - 4  Jan 2000:  Grid interpol. added.  No explicity testing done,
;                 although component testing at least minimally passed.
; - 5  Jan 2000:  Single time slice reading added.  Passed minimally
;                 reasonable tests.
; - 18 Jan 2000:  Add default grid interpol. is no interpol.  Also add
;                 keyword to force no grid_interp.
;-------------------------------------------------------------------------

PRO READDATA, fname,field,NFIELD,delt,NTSKIP, dataout  $
            , IUNIT_OPEN=iunit_open  $
            , FORCE_NO_GRID_INTERP=force_no_grid_interp



;             --- parameter initializing and error checking ---

On_Error, 0
sizeinfo = SIZE(dataout)
case sizeinfo[0] of
  3: begin
      NX = sizeinfo[1]
      NY = sizeinfo[2]
      NT = sizeinfo[3]
      if (sizeinfo[4] ne 5) then Message, 'error--data type bad'
      data = DBLARR(NX,NY,NT)     ;local variable for dataout calcs.
    end
  2: begin
      NX = sizeinfo[1]
      NY = sizeinfo[2]
      NT = 1L                     ;only cycle through one time step
      if (sizeinfo[3] ne 5) then Message, 'error--data type bad'
      data = DBLARR(NX,NY,2L)     ;local variable for dataout calcs.
    end                           ;  extra time slice will be unused
  else: Message, 'error--data dims bad'
endcase


if (NX lt 2) or (NY lt 2) then Message, 'error--NX,NY too small'
if (NFIELD ne 1) and (NFIELD ne 2) and (NFIELD ne 3) and  $
   (NFIELD ne 4) and (NFIELD ne 5) and (NFIELD ne 31) then begin
   Message, 'error--NFIELD incorrect'
endif
case delt of
  1.0:   if (NTSKIP ne 0) and (NTSKIP ne 360) then Message, 'error--NTSKIP'
  30.0:  if (NTSKIP ne 0) and (NTSKIP ne 12)  then Message, 'error--NTSKIP'
  else:  Message, 'delt wrong value'
endcase



;                            --- local arrays ---

tmpdatait = FLTARR(NX,NY,NFIELD)  ;temp. grid 1 timestep data (single prec.)
tmpdata1  = FLTARR(NX,NY)         ;temp. grid 1 field (single prec.)
tmpunits  = 'dummy'               ;dummy string
tmpunits2 = 'dummy'               ;dummy string




;        --- define constants and determine type of data to read in ---

Vartypeit, field,NFIELD, vartype,const1,const2  $
                       , ivar1,ivar2,convmult1,convmult2  $
                       , IVARS=ivar_sing, CONVMULT=convmult_sing  $
                       , UNITS=units_sing, SQUNIT=squnit_sing  $
                       , GRIDLOCTYPE=gridloctype

dc1  = DOUBLE(const1)
dc2  = DOUBLE(const2)
dcc1 = DOUBLE(const1) * DOUBLE(convmult1)
dcc2 = DOUBLE(const2) * DOUBLE(convmult2)



;                           --- read in data ---

if (N_ELEMENTS(iunit_open) ne 0) then begin   ;not-fname mode vs. fname mode
  if (NTSKIP ne 0) then  $
    Print, '     - Note:  NTSKIP ne 0 while IUNIT_OPEN is being used'
  iunit = iunit_open
endif else begin
  Print, '  -> Opening QTCM outputted data file ' + STRTRIM(fname,2)
  Openr, iunit, fname, /F77_UNFORMATTED, /GET_LUN
endelse


case vartype of    ; --> begin vartype case

  "double": begin      ; --> begin vartype double

      if (NFIELD eq 1) then Message, "error--NFIELD must be gt 1"
      if (ivar1 gt NFIELD) then Message, "error--bad ivar1 returned"
      if (ivar2 gt NFIELD) then Message, "error--bad ivar2 returned"
      Print, "--> Using convmult 1 = ",convmult1
      Print, "--> Using convmult 2 = ",convmult2
      ivar1=ivar1-1                       ;convert ivar1 to tmpdatait notation
      ivar2=ivar2-1                       ;convert ivar2 to tmpdatait notation

      if (NTSKIP gt 0) then begin         ;skip first NTSKIP time steps
        for it=1,NTSKIP do begin
          for id=0,NFIELD-1 do Readu, iunit, tmpdata1
        endfor
      endif

      case 1 of
        ((convmult1 eq 1.0) and (convmult2 eq 1.0)): begin
            for it=0L,NT-1L do begin             ; -> read data from QTCM file
              for id=0,NFIELD-1 do begin         ;    in 1 timestep:  no unit
                Readu, iunit, tmpdata1           ;    convert needed.
                tmpdatait[*,*,id] = tmpdata1[*,*]
              endfor
              data[*,*,it] = ( dc1 * DOUBLE(tmpdatait[*,*,ivar1]) )  $
                            +( dc2 * DOUBLE(tmpdatait[*,*,ivar2]) )
            endfor
          end

        else: begin
            for it=0L,NT-1L do begin             ; -> read data from QTCM file
              for id=0,NFIELD-1 do begin         ;    in 1 timestep:  unit
                Readu, iunit, tmpdata1           ;    convert needed.
                tmpdatait[*,*,id] = tmpdata1[*,*]
              endfor
              data[*,*,it] = ( dcc1 * DOUBLE(tmpdatait[*,*,ivar1]) )  $
                            +( dcc2 * DOUBLE(tmpdatait[*,*,ivar2]) )
            endfor
          end
      endcase

    end                ; --> end vartype double


  "single": begin      ; --> begin vartype single

      ivar      = ivar_sing               ;get ivar & convmult
      dconvmult = convmult_sing      
      if (NFIELD eq 1) then ivar=1       ;ivar if only 1 field
      if (ivar gt NFIELD) then Message, "error--bad ivar returned"
      Print, "--> Using single convmult = ",dconvmult
      ivar=ivar-1                         ;convert ivar to tmpdatait notation

      if (NTSKIP gt 0) then begin         ;skip first NTSKIP time steps
        for it=1,NTSKIP do begin
          for id=0,NFIELD-1 do Readu, iunit, tmpdata1
        endfor
      endif

      case 1 of
        (dconvmult eq 1.0): begin
          for it=0L,NT-1 do begin             ;read data from QTCM file
            for id=0,NFIELD-1 do begin        ;  in 1 timestep
              Readu, iunit, tmpdata1
              tmpdatait[*,*,id] = tmpdata1[*,*]
            endfor
            data[*,*,it] = DOUBLE(tmpdatait[*,*,ivar])  ;no need unit convert
          endfor
        end

      else: begin
          for it=0L,NT-1 do begin             ;read data from QTCM file
            for id=0,NFIELD-1 do begin        ;  in 1 timestep
              Readu, iunit, tmpdata1
              tmpdatait[*,*,id] = tmpdata1[*,*]
            endfor
            data[*,*,it] = dconvmult  $      ;unit convert
                          *DOUBLE(tmpdatait[*,*,ivar])
          endfor
        end
      endcase

    end                ; --> end vartype single


  else: Message, "error--field not found"


endcase            ; --> end vartype case

if (N_ELEMENTS(iunit_open) eq 0) then begin   ;close file if fname mode
  Print, '  -> Closing QTCM outputted data file ' + STRTRIM(fname,2)
  Free_Lun, iunit
endif



;                            --- grid interpolate ---

if (KEYWORD_SET(force_no_grid_interp)) then begin
  Print, '  -> Forced no interpol. keyword set'

endif else begin
  case 1 of
    (gridloctype eq 'u') or (gridloctype eq 'v') or (gridloctype eq 'T'):  $
      begin
        tmpinterp_in = DBLARR(NX,NY)
        for it=0L,NT-1L do begin
          tmpinterp_in[*,*] = data[*,*,it]
          tmpinterp_out     = GRID_INTERP(gridloctype, tmpinterp_in)
          data[*,*,it]      = tmpinterp_out[*,*]
        endfor
      end

    else:  Print, '  -> Unrecognized grid type:  Default no interpol.'
  endcase

endelse



;                            --- copy for output ---

case sizeinfo[0] of
  3: dataout[*,*,*] = data[*,*,*]
  2: dataout[*,*]   = data[*,*,0L]
  else: Message, 'error--data dims bad'
endcase



return
end     ;=== end procedure ===



; ========== end file ==========
