; =========================================================================
; $Id: vartypeit.pro,v 1.1 2001/10/22 18:48:46 jlin Exp $
;
; File Description:  This file contains only the procedure vartypeit
; =========================================================================


;--------------------------------------------------------------------------
;                          procedure vartypeit
;
; Purpose:
;   For the field varname, tells what sort of vartype the variable
;   has, and constants (when multiple fields are used as input),
;   if appropriate.  Also returns ivars, convmult, units, and squnit
;   for the single/composite field if these keyword arguments are
;   present.
;
;   Philosophy:  I try to control as much of the secondary field values
;   as I can via VARTYPEIT.  I try to control as much of the primary 
;   field values as I can via VARLIST.
;
;   NB1:  If vartype is 'double', and NFIELD ne 1, then ivars (if
;   defined) is returned as -999.
;
;   NB2:  If vartype is 'single', const1,2 and convmult1,2 are set
;   to NaN, and ivar1,2 is set to -999, since they don't apply.
;
;   NB3:  For all secondary fields, vartypeit returns gridloctype
;   (if selected) that is either 'u', 'v', or 'T'.  For non-secondary
;   fields, gridloctype is determined by a call to Varlist.
;
; Parameters In:
;   varname       name of variable (string)
;   NFIELD        number of fields (integer)
;
; Parameters Out:
;   vartype       Has value 'double' if requires two QTCM1 output
;                 variables to calculate, and value 'single' if not;
;                 if NFIELD=1, vartype is automatically 'single'
;                 (string)
;   const1        const1 constant (double)
;   const2        const2 constant (double)
;   ivar1,2       Number of variable in list (integer)
;   convmult1,2   Units conversion multiplier for secondary field
;                 constituents (double)
;
; Keywords Passed (optional unless otherwise noted):
;   ivars         Number of variable in list (integer); this output
;                 is designed for the case of vartype = 'single'
;   convmult      Units conversion multiplier for the field, not
;                 just the constituents of the field (double); for
;                 secondary fields, this is set to convmult2
;   units         Units of variable, after convert (string)
;   squnit        Units of square of variable, after convert (string)
;   gridloctype   Type of the location of the grid; 'u', 'v', or 'T'
;                 on the staggered Arakawa C-grid (string).
;
; Notes:
; - User-written procedures used:  Varlist
; - No common blocks used in this procedure.
; - Most NFIELD checks are found in Varlist.  It is very important
;   that the correct Varlist is used.
; - Works with QTCM v2.1 standard output.
; - Written in IDL 5.0.
;
; Revision History:
; - 29 Dec 1999:  Orig. by Johnny Lin, UCLA Dept. of Atmos. Sci.
;                 Based on QTCM1 v2.0 tools vartypeit.pro RCSv1.12.
;                 Passed reasonable tests.
; - 4  Jan 2000:  GRIDLOCTYPE added.  Passed minimally reasonable tests.
;--------------------------------------------------------------------------

PRO VARTYPEIT, varname,NFIELD, vartype,const1,const2   $
                             , ivar1,ivar2,convmult1,convmult2  $
                             , IVARS=ivars, CONVMULT=convmult  $
                             , UNITS=units, SQUNIT=squnit  $
                             , GRIDLOCTYPE=gridloctype



;        --- on-entrance error checks and undefined value setting ---

On_Error, 0
sizeinfo1 = SIZE(varname)
sizeinfo2 = SIZE(NFIELD)
if (sizeinfo1[1] ne 7) then Message, 'error-varname wrong type'
if (sizeinfo2[1] ne 2) then Message, 'error-NFIELD wrong type'



;                     --- default "undefined" values ---

vartype = 'dummy'
tmpivar = -999
tmpconvmult = !VALUES.F_NAN
tmpunits  = 'dummy'
tmpsqunit = 'dummy'

if (ARG_PRESENT(ivars) eq 1)       then ivars=-999
if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='Not Given'

ivar1     = -999            &   ivar2     = -999
const1    = !VALUES.F_NAN   &   const2    = !VALUES.F_NAN
convmult1 = !VALUES.F_NAN   &   convmult2 = !VALUES.F_NAN



;                        --- define constants ---

; Naming scheme:  Wind const. for V1x, where x is the level in mb.  Taken
; from qtcmpar.gs utility in the QTCM1 v2.1 standard distribution.  As of
; 29 Dec 1999.

V11000 =  -0.235772d0
V1950  =  -0.2204077d0
V1900  =  -0.2044558d0
V1850  =  -0.187438d0
V1800  =  -0.168727d0
V1700  =  -0.124817d0
V1600  =  -6.869278d-02
V1500  =   7.053178d-03
V1400  =   0.117307d0
V1300  =   0.2932457d0
V1200  =   0.5870798d0

V1s    =  V11000   ;surf. wind const. set to 1000 mb value
V1t    =  V1200    ;TOA wind const. set to 200 mb value

 
 
;        --- determine type of data to read in (vartype and const) ---

; Notes:  Primary fields are fields outputted by the QTCM1 model directly.
; Secondary fields are composites of primary fields.  However, secondary
; fields can be vartype 'single' if I'm processing a file that contains a
; precalculated secondary field.  To check for that case, I assume and
; enforce that any file with precalculated secondary fields has NFIELD=1.

case varname of

; --> list of secondary fields
  "us": begin
      Varlist, 'u1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'u0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1s & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='u'
    end
  "ut": begin
      Varlist, 'u1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'u0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1t & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='u'
    end
  "u850": begin
      Varlist, 'u1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'u0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1850 & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='u'
    end
  "u200": begin
      Varlist, 'u1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'u0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1200 & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='u'
    end

  "vs": begin
      Varlist, 'v1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'v0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1s & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='v'
    end
  "vt": begin
      Varlist, 'v1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'v0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1t & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='v'
    end
  "v850": begin
      Varlist, 'v1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'v0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1850 & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='v'
    end
  "v200": begin
      Varlist, 'v1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'v0',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=V1200 & const2=1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='v'
    end

  "q_T": begin
      Varlist, 'q1',ivar1,convmult1,tmpunits,tmpsqunit, NFIELD=NFIELD
      Varlist, 'T1',ivar2,convmult2,tmpunits,tmpsqunit, NFIELD=NFIELD
      const1=1.d0 & const2=-1.0d0 & vartype='double'
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype='T'
    end

; --> this option is selected if the field is not a secondary field
  else: begin         
      Varlist, varname,tmpivar,tmpconvmult,tmpunits,tmpsqunit  $
             , NFIELD=NFIELD, GRIDLOCTYPE=tmptype
      if (ARG_PRESENT(ivars) eq 1)       then ivars=tmpivar
      if (ARG_PRESENT(convmult) eq 1)    then convmult=tmpconvmult
      if (ARG_PRESENT(units) eq 1)       then units=tmpunits
      if (ARG_PRESENT(squnit) eq 1)      then squnit=tmpsqunit
      if (ARG_PRESENT(gridloctype) eq 1) then gridloctype=tmptype
      vartype='single'
    end

endcase


; --> assign convmult, units, squnit keywords for secondary units
if (vartype eq 'double') then begin
  if (ARG_PRESENT(convmult) eq 1) then convmult=convmult2
  if (ARG_PRESENT(units) eq 1)    then units=tmpunits
  if (ARG_PRESENT(squnit) eq 1)   then squnit=tmpsqunit
endif



;                          --- pre-exit setting ---

if (NFIELD eq 1) then begin
  if (ARG_PRESENT(ivars) eq 1) then ivars=1   ;force for NFIELD=1
  vartype='single'                            ;force for NFIELD=1
endif

if (vartype eq 'single') then begin
  ivar1     = -999            &   ivar2     = -999
  const1    = !VALUES.F_NAN   &   const2    = !VALUES.F_NAN
  convmult1 = !VALUES.F_NAN   &   convmult2 = !VALUES.F_NAN
endif



RETURN
END     ;=== end procedure ===


; ========== end file ==========
